/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.model;

import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import org.openide.nodes.Node;

/**
 *
 * @author Wytse Visser
 */
public class MachineProperty<T> implements Serializable {

    private Class<T> valueType;
    private String name;
    private T value;
    private String[] possibleValues;
    
    public MachineProperty(Class<T> valueType, String name, T defaultValue) {
        this.valueType = valueType;
        this.name = name;
        this.value = defaultValue;
    }
    
    public T getValue() throws IllegalAccessException, InvocationTargetException {
        return value;
    }

    public final void setValue(T val) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException {
        this.value = val;
    }

    public void setPossibleValues(String[] possibleValues) {
        this.possibleValues = possibleValues;
    }
    
    public String getName() {
        return this.name;
    }

    public Node.Property<T> getProperty() {
        return new Node.Property<T>(MachineProperty.this.valueType) {

            @Override
            public boolean canRead() {
                return true;
            }

            @Override
            public T getValue() throws IllegalAccessException, InvocationTargetException {
                return value;
            }

            @Override
            public boolean canWrite() {
                return true;
            }

            @Override
            public void setValue(T val) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException {
                MachineProperty.this.value = val;
            }

            @Override
            public String getName() {
                return MachineProperty.this.getName();
            }

            @Override
            public PropertyEditor getPropertyEditor() {
                final String[] tags = possibleValues;
                if (tags == null || tags.length == 0) {
                    return super.getPropertyEditor();
                } else {
                    return new PropertyEditorSupport() {

                        @Override
                        public String[] getTags() {
                            return tags;
                        }
                    };
                }
            }
        };
    }
}