/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.model;

import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.ItemIdType;
import org.openide.nodes.Node;
import org.openide.nodes.Sheet;

/**
 *
 * @author Wytse Visser
 */
public class Vehicle extends Machine {

    private static final Logger LOGGER = Logger.getLogger(Vehicle.class.getName());
    private transient VehicleGeographical vehicleGeographical;
    private int horsepower;
    private int fuelTankCapacity;
    private double displacement;
    public static final String PROP_HORSEPOWER = "horsepower";
    public static final String PROP_DISPLACEMENT = "displacement";
    public static final String PROP_FUELTANKCAPACITY = "fuelTankCapacity";
    public static final ItemIdType ITEM_ID_TYPE = ItemIdType.VCE;
    public static final String EXT = "vce";

    public Vehicle(EnterpriseIdProvider eip, String name) {
        super(eip, ITEM_ID_TYPE, name);
    }

    /**
     * Get the value of fuelTankCapacity
     *
     * @return the value of fuelTankCapacity
     */
    public int getFuelTankCapacity() {
        return fuelTankCapacity;
    }

    /**
     * Set the value of fuelTankCapacity
     *
     * @param fuelTankCapacity new value of fuelTankCapacity
     */
    public void setFuelTankCapacity(int fuelTankCapacity) {
        int oldFuelTankCapacity = this.fuelTankCapacity;
        this.fuelTankCapacity = fuelTankCapacity;
        getPropertyChangeSupport().firePropertyChange(PROP_FUELTANKCAPACITY, oldFuelTankCapacity, fuelTankCapacity);
    }

    /**
     * Get the value of displacement
     *
     * @return the value of displacement
     */
    public double getDisplacement() {
        return displacement;
    }

    /**
     * Set the value of displacement
     *
     * @param displacement new value of displacement
     */
    public void setDisplacement(double displacement) {
        double oldDisplacement = this.displacement;
        this.displacement = displacement;
        getPropertyChangeSupport().firePropertyChange(PROP_DISPLACEMENT, oldDisplacement, displacement);
    }

    /**
     * Get the value of horsepower
     *
     * @return the value of horsepower
     */
    public int getHorsepower() {
        return horsepower;
    }

    /**
     * Set the value of horsepower
     *
     * @param horsepower new value of horsepower
     */
    public void setHorsepower(int horsepower) {
        int oldHorsepower = this.horsepower;
        this.horsepower = horsepower;
        getPropertyChangeSupport().firePropertyChange(PROP_HORSEPOWER, oldHorsepower, horsepower);
    }

    @Override
    public MachineGeometrical getGeometrical() {
        if (vehicleGeographical == null) {
            vehicleGeographical = new VehicleGeographical();
        }
        return vehicleGeographical;
    }

    @Override
    public Sheet createSheet() {
        Sheet sheet = super.createSheet();
        try {
            Node.Property[] properties = new Node.Property[]{
                new ReflectionProperty(int.class, PROP_HORSEPOWER, "Horsepower"),
                new ReflectionProperty(double.class, PROP_DISPLACEMENT, "Displacement"),
                new ReflectionProperty(int.class, PROP_FUELTANKCAPACITY, "Fuel tank capacity")
            };
            for (Node.Property property : properties) {
                sheet.get(Sheet.PROPERTIES).put(property);
            }
        } catch (NoSuchMethodException ex) {
            LOGGER.log(Level.WARNING, "Could no create propertySheet for vehicle", ex.getMessage());
        }

        return sheet;
    }

    private class VehicleGeographical extends MachineGeometrical {

        public VehicleGeographical() {
            super(Vehicle.this);
        }

        @Override
        public String getIconPath() {
            return "nl/cloudfarming/client/fleet/vehicle/steering_wheel_332.png";
        }
    }
}
