/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.model.gps;

import java.awt.event.ActionEvent;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Action;
import nl.cloudfarming.client.fleet.model.Machine;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * This action creates a GPS tracker for its machine and provides the ability to
 * enable and disable it.
 *
 * @author Wytse Visser
 */
@NbBundle.Messages({"enable_gps_tracking=Enable gps tracking", "disable_gps_tracking=Disable gps tracking"})
public class GPSAction extends AbstractAction {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.fleet.model.gps.GPSAction");
    private Machine machine;
    private GPSTracker tracker;
    private static Map<Machine, GPSAction> actionMap;

    protected GPSAction(Machine machine) {
        this.machine = machine;
        updateName();
    }

    /**
     * Get the instance of the GPSAction for the given machine.
     *
     * @param machine The machine
     * @return The instance of the GPSAction for the given machine
     */
    public static Action get(Machine machine) {
        GPSAction action = getActionMap().get(machine);
        if (action == null) {
            action = new GPSAction(machine);
            actionMap.put(machine, action);
        }
        action.setEnabled(action.getGPSTracker() != null);
        return action;
    }

    private static Map<Machine, GPSAction> getActionMap() {
        if (actionMap == null) {
            actionMap = new HashMap<>();
        }
        return actionMap;
    }

    protected GPSTracker getGPSTracker() {
        if (tracker == null) {
            TrackerFactory factory = Lookup.getDefault().lookup(TrackerFactory.class);
            if (factory != null) {
                tracker = factory.createTracker();
                tracker.setMachine(machine);
            } else {
                LOGGER.warning("No GPSTracker installed.");
            }
        }
        return tracker;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        if (getGPSTracker() != null) {
            LOGGER.finest("GPSTRACKER != null");
            getGPSTracker().toggle();
            updateName();
        } else {
            LOGGER.finest("GPSTRACKER == null");
        }
    }

    private void updateName() {
        if (getGPSTracker() != null && getGPSTracker().isEnabled()) {
            putValue(NAME, Bundle.disable_gps_tracking());
        } else {
            putValue(NAME, Bundle.enable_gps_tracking());
        }
    }
}
